@@ -23,7 +23,8 @@
 *  with suitably chosen file names. Each such file will be
 *  individually playable. (Gaps?)
 *
-* Copyright (c) 1995-1998 by H. Schulzrinne (Columbia U.)
+* Copyright (c) 1995-2000 by H. Schulzrinne and Ping Pan (Columbia University)
+*   All Rights Reserved
 */
 
 #include <sys/types.h>
@@ -85,7 +86,7 @@ static void hex(char *buf, int len)
   int i;
 
   for (i = 0; i < len; i++) {
-    printf("%02x ", (unsigned char)buf[i]);
+    printf("%02x", (unsigned char)buf[i]);
   }
   printf("\n");
 } /* hex */
@@ -237,7 +238,8 @@ static int parse_header(char *buf)
 static int parse_data(char *buf, int len)
 {
   rtp_hdr_t *r = (rtp_hdr_t *)buf;
-  int i;
+  rtp_hdr_ext_t *ext;
+  int i, ext_len;
   int hlen = 0;
 
   /* Show vat format packets. */
@@ -255,7 +257,7 @@ static int parse_data(char *buf, int len)
       return hlen;
     }
     printf(
-    "v=%d p=%d x=%d cc=%d m=%d pt=%d (%s,%d,%d) seq=%u ts=%lu ssrc=0x%lx\n",
+    "v=%d p=%d x=%d cc=%d m=%d pt=%d (%s,%d,%d) seq=%u ts=%lu ssrc=0x%lx ",
       r->version, r->p, r->x, r->cc, r->m, 
       r->pt, pt_map[r->pt].enc, pt_map[r->pt].ch, pt_map[r->pt].rate,
       ntohs(r->seq), 
@@ -264,6 +266,22 @@ static int parse_data(char *buf, int len)
     for (i = 0; i < r->cc; i++) {
       printf("csrc[%d] = %0x ", i, r->csrc[i]);
     }
+    if (r->x) {  /* header extension */
+      ext = (rtp_hdr_ext_t *)((char *)buf + hlen);
+      ext_len = ntohs(ext->len);
+
+      printf("ext_type=0x%x ", ntohs(ext->ext_type));
+      printf("ext_len=%d ", ext_len);
+
+      if (ext_len) {
+        printf("ext_data=");
+        hex((char *)(ext+1), (ext_len*4));
+      }
+      else
+        printf("\n");
+    }
+    else
+      printf("\n");
   }
   else {
     printf("RTP version wrong (%d).\n", r->version);
@@ -390,14 +408,14 @@ static int parse_control(char *buf, int len)
           (unsigned long)ntohl(r->r.rr.ssrc),
           r->common.p, r->common.count,
 	      ntohs(r->common.length));
-        printf("ntp=%lu.%lu ts=%lu psent=%lu osent=%lu\n",
+        printf("  ntp=%lu.%lu ts=%lu psent=%lu osent=%lu\n",
           (unsigned long)ntohl(r->r.sr.ntp_sec),
           (unsigned long)ntohl(r->r.sr.ntp_frac),
           (unsigned long)ntohl(r->r.sr.rtp_ts),
           (unsigned long)ntohl(r->r.sr.psent),
           (unsigned long)ntohl(r->r.sr.osent));
         for (i = 0; i < r->common.count; i++) {
-          printf("  (ssrc=%0lx fraction=%g lost=%lu last_seq=%lu jit=%lu lsr=%lu dlsr=%lu)\n",
+          printf("  (ssrc=0x%lx fraction=%g lost=%lu last_seq=%lu jit=%lu lsr=%lu dlsr=%lu )\n",
            (unsigned long)ntohl(r->r.sr.rr[i].ssrc),
            r->r.sr.rr[i].fraction / 256.,
            (unsigned long)ntohl(r->r.sr.rr[i].lost), /* XXX I'm pretty sure this is wrong */
@@ -414,7 +432,7 @@ static int parse_control(char *buf, int len)
           (unsigned long)ntohl(r->r.rr.ssrc), r->common.p, r->common.count,
 	      ntohs(r->common.length));
         for (i = 0; i < r->common.count; i++) {
-          printf("(ssrc=%0lx fraction=%g lost=%lu last_seq=%lu jit=%lu lsr=%lu dlsr=%lu)\n",
+          printf("  (ssrc=0x%lx fraction=%g lost=%lu last_seq=%lu jit=%lu lsr=%lu dlsr=%lu )\n",
             (unsigned long)ntohl(r->r.rr.rr[i].ssrc),
             r->r.rr.rr[i].fraction / 256.,
             (unsigned long)ntohl(r->r.rr.rr[i].lost),
@@ -454,14 +472,15 @@ static int parse_control(char *buf, int len)
         printf(" (BYE p=%d count=%d len=%d\n", 
           r->common.p, r->common.count, ntohs(r->common.length));
         for (i = 0; i < r->common.count; i++) {
-          printf("ssrc[%d]=%0lx ", i, 
+          printf("  (ssrc[%d]=0x%0lx ", i, 
             (unsigned long)ntohl(r->r.bye.src[i]));
         }
+        printf(")\n");
         if (ntohs(r->common.length) > r->common.count) {
           buf = (char *)&r->r.bye.src[r->common.count];
           printf("reason=\"%*.*s\"", *buf, *buf, buf+1); 
         }
-        printf(")\n");
+        printf(" )\n");
         break;
 
       /* invalid type */
